/*
 * File         : Jacobi.C
 * Author       : Guansong Zhang
 * Created      : Tue Feb  3 13:35:51 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */


#include "math.h"
#include <stdio.h>
#include "ad++.h"


const int n = 16 ;

int main(int argc, char* argv[]) {
  AdlibInit(argc, argv) ;

  Procs2 p(1, 1) ;

  ON(p) {
    const int w [2] = {1, 1} ;
    BlockRange x(n, p.dim(0), LAY_EXTENDED, 1, 1) ;
    BlockRange y(n, p.dim(1), LAY_EXTENDED, 1, 1) ;
    Array2<float> u(x, y) ;

    Index i(x), j(y) ;
    OVERALL(i) {
      OVERALL(j) {
        if(i == 0 || i == n - 1 || j == 0 || j == n - 1)
          u(i, j) = 1.0 ;
        else
          u(i, j) = 0.0 ;
      } ALLOVER(j) ;
    } ALLOVER(i) ;


    Index ic(x.subrng(n - 2, 1)), jc(y.subrng(n - 2, 1)) ;
    Array2<float> uold(x, y) ;

    for(int iter = 0 ; iter < 100 ; iter++) {
      copy(uold, u) ;
      writeHalo(uold, w, w) ;

      if (iter==10) 
	AdlibClock(0);

      /* 
      OVERALL(ic) {
        OVERALL(jc) {
          u(ic, jc) = 0.25 * (uold(ic << 1, jc) + uold(ic >> 1, jc) +
                              uold(ic, jc << 1) + uold(ic, jc >> 1)) ;
        } ALLOVER(jc) ;
      } ALLOVER(ic) ;
      */

      float * u_ptr = u.dat();
      float * uold_ptr = uold.dat();

      apgStack.push(&apg) ;
      apg.restrict(jc.dim); 
      for (jc.beginLocBlk(); jc.test(); jc.nextLocBlk()) {
	int u1_bas=u.str(1).offset(jc);
	const int u1_stp=u.str(1).step(jc);
	const int u1_len=u.str(1).len();

	int uold1_bas=uold.str(1).offset(jc);
	const int uold1_stp=uold.str(1).step(jc);
	const int uold1_len=uold.str(1).len();

	for (int jc_blk=0; jc_blk<jc.count; jc_blk++) {
	  apgStack.push(&apg) ;
	  apg.restrict(ic.dim);
	  for (ic.beginLocBlk(); ic.test(); ic.nextLocBlk()) {
	    int u0_bas=u.str(0).offset(ic);
	    const int u0_stp=u.str(0).step(ic);
	    const int u0_len=u.str(0).len();
	    
	    int uold0_bas=uold.str(0).offset(ic);
	    const int uold0_stp=uold.str(0).step(ic);
	    const int uold0_len=uold.str(0).len();
	    
	    for (int ic_blk=0; ic_blk<ic.count; ic_blk++) {
	  
	      int u_bas=u0_bas+u1_bas;
	      int uold_bas=uold0_bas+uold1_bas;

	      u_ptr[u_bas]
		  =(uold_ptr[uold_bas-uold0_len]+
		    uold_ptr[uold_bas+uold0_len]+
 		    uold_ptr[uold_bas-uold1_len]+
 		    uold_ptr[uold_bas+uold1_len])*(float)0.25;

	      u0_bas+=u0_stp;
	      uold0_bas+=uold0_stp;

	    }

	  } apgStack.pop(&apg);

	  u1_bas+=u1_stp;
	  uold1_bas+=uold1_stp;

	}

      } apgStack.pop(&apg);

    }


    AdlibClock(1);

    //gprintf("iteration = %d\n", iter) ;
    aprintf("%7.3f%*N", u, n) ;

  } NO(p) ;

  AdlibFinalize() ;
}
